%{
/*
scan.l - parser for Java, compile with Lex or Flex
Copyright  (C) 2000 Hugues Johnson

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software 
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include"expresso_tab.c"

void yyerror(char *s);
int Instructions;
int Lines;

char *TokenName[99]={   "EOF","abstract","boolean","break","byte","case",
                        "catch","char","class","const","continue","default",
                        "do","double","else","extends","false","final",
                        "finally","float","for","goto","if","implements",
                        "import","instanceof","int","interface","long",
                        "native","new","null","package","private","protected",
                        "public","return","short","static","super","switch",
                        "synchronized","this","throw","throws","transient",
                        "true","try","void","volatile","while","literal",
                        "identifier","(",")","{","}","[","]",";",",",".",
                        "=",">","<","!","~","?",":","==","<=",">=","!=",
                        "||","&&","++","--","+","-","*","/","&","|","^","%","<<",
                        ">>",">>>","+=","-=","*=","/=","&=","|=","^=",
                        "%=","<<=",">>=",">>>=" };
%}

%%
[\t]+ ;
[\n]                            { Lines++; }

"abstract"                      { return(ABSTRACT); }              
"boolean"                       { return(BOOLEAN); }
"break"                         { Instructions++; return(BREAK); }
"byte"                          { return(BYTE); }
"case"                          { return(CASE); }
"catch"                         { Instructions++; return(CATCH); }
"char"                          { return(CHAR); }
"class"                         { return(CLASS); }
"const"                         { return(CONST); }
"continue"                      { Instructions++; return(CONTINUE); }
"default"                       { return(DEFAULT); }
"double"                        { return(DOUBLE); }
"do"                            { Instructions++; return(DO); }
"else"                          { Instructions++; return(ELSE); }
"extends"                       { return(EXTENDS); }
"false"                         { return(FALSE); }
"final"                         { return(FINAL); }
"finally"                       { return(FINALLY); }
"float"                         { return(FLOAT); }
"for"                           { Instructions++; return(FOR); }
"goto"                          { Instructions++; return(GOTO); }
"if"                            { Instructions++; return(IF); }
"implements"                    { return(IMPLEMENTS); }
"import"                        { Instructions++; return(IMPORT); }
"instanceof"                    { return(INSTANCEOF); }
"int"                           { return(INT); }
"interface"                     { return(INTERFACE); }
"long"                          { return(LONG); }
"native"                        { return(NATIVE); }
"new"                           { return(NEW); }
"null"                          { return(NULLTOKEN); }
"package"                       { return(PACKAGE); }
"private"                       { return(PRIVATE); }
"protected"                     { return(PROTECTED); }
"public"                        { return(PUBLIC); }
"return"                        { Instructions++; return(RETURN); }
"short"                         { return(SHORT); }
"static"                        { return(STATIC); }
"super"                         { return(SUPER); }
"switch"                        { Instructions++; return(SWITCH); }
"synchronized"                  { return(SYNCHRONIZED); }
"this"                          { return(THIS); }
"throw"                         { Instructions++; return(THROW); }
"throws"                        { return(THROWS); }
"transient"                     { return(TRANSIENT); }
"true"                          { return(TRUE); }
"try"                           { Instructions++; return(TRY); }
"void"                          { return(VOID); }
"volatile"                      { return(VOLATILE); }
"while"                         { Instructions++; return(WHILE); }

"("                             { return(LPAREN); }
")"                             { return(RPAREN); }
"{"                             { return(LBRACE); }
"}"                             { return(RBRACE); }
"["                             { return(LSQUARE); }
"]"                             { return(RSQUARE); }
";"                             { return(SEMICOLON); }
","                             { return(COMMA); }
"."                             { return(PERIOD); }

">>>="                          { Instructions++; return(RIGHTSHIFTZEROASSIGN); }
"<<="                           { Instructions++; return(LEFTSHIFTASSIGN); }
">>="                           { Instructions++; return(RIGHTSHIFTASSIGN); }
">>>"                           { Instructions++; return(RIGHTSHIFTZERO); }
"<<"                            { Instructions++; return(LEFTSHIFT); }
">>"                            { Instructions++; return(RIGHTSHIFT); }
"+="                            { Instructions++; return(PLUSASSIGN); }
"-="                            { Instructions++; return(MINUSASSIGN); }
"*="                            { Instructions++; return(MULTIPLYASSIGN); }
"/="                            { Instructions++; return(DIVIDEASSIGN); }
"&="                            { Instructions++; return(CONCATENATEASSIGN); }
"|="                            { Instructions++; return(SINGLEBARASSIGN); }
"^="                            { Instructions++; return(CARROTASSIGN); }
"%="                            { Instructions++; return(MODULUSASSIGN); }
"=="                            { Instructions++; return(EQUAL); }
"<="                            { Instructions++; return(LESSTHANEQUAL); }
">="                            { Instructions++; return(GREATERTHANEQUAL); }
"!="                            { Instructions++; return(NOTEQUAL); }
"||"                            { Instructions++; return(OR); }
"&&"                            { Instructions++; return(AND); }
"++"                            { Instructions++; return(INCREMENT); }
"--"                            { Instructions++; return(DECREMENT); }
"="                             { Instructions++; return(ASSIGNMENT); }
">"                             { Instructions++; return(GREATERTHAN); }
"<"                             { Instructions++; return(LESSTHAN); }
"!"                             { Instructions++; return(UNARYNOT); }
"~"                             { Instructions++; return(TILDE); }
"?"                             { Instructions++; return(QUESTION); }
":"                             { Instructions++; return(COLON); }
"+"                             { Instructions++; return(PLUS); }
"-"                             { Instructions++; return(MINUS); }
"*"                             { Instructions++; return(MULTIPLY); }
"/"                             { Instructions++; return(DIVIDE); }
"&"                             { Instructions++; return(CONCATENATE); }
"|"                             { Instructions++; return(SINGLEBAR); }
"^"                             { Instructions++; return(CARROT); }
"%"                             { Instructions++; return(MODULUS); }

[1-9][0-9]*                     { yylval.intval=atoi(yytext); return(LITERAL); }
[a-zA-Z_$][a-zA-Z_$0-9]*        { yylval.identname=strdup(yytext); return(IDENTIFIER); }
%%

#include<stdio.h>
#include<string.h>

yyerror(char *s)
{ printf("%s %s\n",s,yytext); }

main(argc,argv)
int argc;
char **argv;
{
        int val;
        /* keep filenames in DOS 8.3 format */
        char ListFile[12];
        char CodeFile[12];
        FILE *stream;

        ++argv, --argc;
        Lines=0; Instructions=0;

        switch(argc)
        {
                case 1: /* create default .lis and .cod files */
                        strcpy(ListFile,"default.lis");
                        strcpy(CodeFile,"default.cod");
                        yyin=fopen(argv[0],"r");
                        break;
                case 2: /* create default .cod file */ 
                        strcpy(ListFile,argv[1]);
                        strcpy(CodeFile,"default.cod");
                        yyin=fopen(argv[0],"r");
                        break;
                case 3: /* received all parameters */
                        strcpy(ListFile,argv[1]);
                        strcpy(CodeFile,argv[2]);
                        yyin=fopen(argv[0],"r");
                        break;
                default: /* zero or incorrect number-o-parameters */
                        strcpy(ListFile,"default.lis");
                        strcpy(CodeFile,"default.cod");
                        yyin=stdin;
                        break;
        }
        while(val=yylex()) printf("Wow! It a(n) %s token!\n",TokenName[val]);

        printf("Lines.... %d\n",Lines);
        printf("Instructions.... %d\n",Instructions);
}
